@@ -27,6 +27,7 @@
 #include "xkbcomp.h"
 #include "tokens.h"
 #include "expr.h"
+#include "parseutils.h"
 
 #include <X11/keysym.h>
 #include <X11/Xutil.h>
@@ -39,10 +40,6 @@
 #include "misc.h"
 #include "alias.h"
 
-extern Atom tok_ONE_LEVEL;
-extern Atom tok_TWO_LEVEL;
-extern Atom tok_KEYPAD;
-
 /***====================================================================***/
 
 #define	RepeatYes	1
@@ -931,7 +928,7 @@ GetGroupIndex(KeyInfo * key,
 static Bool
 AddSymbolsToKey(KeyInfo * key,
                 XkbDescPtr xkb,
-                char *field,
+                const char *field,
                 ExprDef * arrayNdx, ExprDef * value, SymbolsInfo * info)
 {
     unsigned ndx, nSyms;
@@ -968,8 +965,12 @@ AddSymbolsToKey(KeyInfo * key,
         return False;
     }
     key->symsDefined |= (1 << ndx);
-    memcpy((char *) key->syms[ndx], (char *) value->value.list.syms,
-           nSyms * sizeof(KeySym));
+    for (i = 0; i < nSyms; i++) {
+        if (!LookupKeysym(value->value.list.syms[i], &key->syms[ndx][i])) {
+            WSGO1("Could not resolve keysym %s\n", value->value.list.syms[i]);
+            key->syms[ndx][i] = NoSymbol;
+        }
+    }
     for (i = key->numLevels[ndx] - 1;
          (i >= 0) && (key->syms[ndx][i] == NoSymbol); i--)
     {
@@ -981,7 +982,7 @@ AddSymbolsToKey(KeyInfo * key,
 static Bool
 AddActionsToKey(KeyInfo * key,
                 XkbDescPtr xkb,
-                char *field,
+                const char *field,
                 ExprDef * arrayNdx, ExprDef * value, SymbolsInfo * info)
 {
     register int i;
@@ -1117,7 +1118,7 @@ static LookupEntry rgEntries[] = {
 static Bool
 SetSymbolsField(KeyInfo * key,
                 XkbDescPtr xkb,
-                char *field,
+                const char *field,
                 ExprDef * arrayNdx, ExprDef * value, SymbolsInfo * info)
 {
     Bool ok = True;
@@ -1234,7 +1235,7 @@ SetSymbolsField(KeyInfo * key,
              uStrCasePrefix("permanentoverlay", field))
     {
         Bool permanent = False;
-        char *which;
+        const char *which;
         int overlayNdx;
         if (uStrCasePrefix("permanent", field))
         {
@@ -1709,7 +1710,7 @@ FindKeyForSymbol(XkbDescPtr xkb, KeySym sym, unsigned int *kc_rtrn)
             if (j < (int) XkbKeyNumSyms(xkb, i))
             {
                 gotOne = True;
-                if ((XkbKeySym(xkb, i, j) == sym))
+                if (XkbKeySym(xkb, i, j) == sym)
                 {
                     *kc_rtrn = i;
                     return True;
@@ -2029,7 +2030,7 @@ CopySymbolsDef(XkbFileInfo * result, KeyInfo * key, int start_from)
         type = &xkb->map->types[types[i]];
         if (type->num_levels < key->numLevels[i])
         {
-            if (warningLevel > 0)
+            if (warningLevel > 5)
             {
                 WARN4
                     ("Type \"%s\" has %d levels, but %s has %d symbols\n",
